from eureqa.analysis.components.base import _Component
from eureqa.utils.jsonrest import _JsonREST
from eureqa.search_settings import SearchSettings

class SearchBuilderLink(_Component):
    """
    Create a SearchBuilderLink component

    Example of using HtmlBlock with a SearchBuilderLink::

        # include a link to the search builder with nothing pre-populated
        search_builder_link = SearchBuilderLink()
        h = HtmlBlock('Click this link to open the pre-populated search builder: {0}'.format(analysis.html_ref(search_builder_link)))
        layout.add_component(h)

        # include a link to the search builder with settings pre-populated by a search template
        datasource = e.create_data_source("data_source_1", "tests/Nutrition.csv")
        search_builder_link = SearchBuilderLink(datasource=datasource,
                                                search_template=search_templates.SearchTemplates.Timeseries,
                                                min_delay=1,
                                                max_delay=10,
                                                target_variable='Calories',
                                                input_variables=['Steps', 'Weight', '(Protein (g))', '(Fat (g))'])
        h = HtmlBlock('Click this link to open the pre-populated search builder: {0}'.format(analysis.html_ref(search_builder_link)))
        layout.add_component(h)

    :param str link_text: the visible, clickable text for the link
    :param DataSource datasource: datasource to pre-populate the search builder with
    :param SearchTemplate search_template: search template information to pre-populate the search builder with
    :param int min_delay: the minimum delay used in a timeseries search
    :param int max_delay: the maximum delay used in a timeseries search
    :param str target_variable: the variable to search for models of
    :param str[] input_variables: the variables to use as inputs in the models
    """

    _component_type_str = 'SEARCH_BUILDER_LINK'

    def __init__(self,
                 link_text=None,
                 datasource=None,
                 search_template=None,
                 min_delay=None,
                 max_delay=None,
                 target_variable=None,
                 input_variables=None,
                 _analysis=None,
                 _component_id=None,
                 _component_type=None):

        # implementation details:
        # self.datasource is a property, property
        # self._datasource is the actual datasource object (cache of the datasource) which may or may not be fetched
        # self._datasource_id a string, and its setting/handling is handled by the JsonRest implementation'

        if link_text is not None:
            self._link_text = link_text
        else:
            self._link_text = "Search Builder"

        datasource_id = datasource._datasource_id if datasource is not None else None
        self._search_template_detail = SearchSettings(datasource_id=datasource_id,
                                                      search_template_id=search_template,
                                                      default_min_delay=min_delay,
                                                      maximum_history_absolute_rows=max_delay,
                                                      target_variable=target_variable,
                                                      input_variables=input_variables)

        super(SearchBuilderLink, self).__init__(_analysis=_analysis, _component_id=_component_id, _component_type=_component_type)


    @property
    def link_text(self):
        """The HTML text which will be wrapped with a link to the search builder.

        :rtype: str
        """
        return getattr(self, "_link_text", None)

    def _to_json(self):
        base = super(SearchBuilderLink, self)._to_json()
        if hasattr(self, '_link_text'): base['link_text'] = self._link_text
        base['search_detail'] = None
        if hasattr(self, '_search_template_detail'): base['search_detail'] = self._search_template_detail._to_json()
        return base

    def _from_json(self, body):
        super(SearchBuilderLink, self)._from_json(body)
        if 'link_text' in body:
            self._link_text = body['link_text']
        if 'search_detail' in body:
            self._search_template_detail = SearchSettings.from_json(None, body['search_detail'])



## Command line interface to analysis runner
import sys
import os

import analysis_template_runner

# Note this string in stdout is used by the template launcher to signal a valid startup. Don't change!
print "Nutonian Analysis Template Launcher"
sys.stdout.flush() # ensure our watcher sees it

runner = analysis_template_runner.analysis_template_runner()

runner.run_args(sys.argv[1:])



from eureqa.analysis.components.base import _Component
from eureqa.utils.jsonrest import _JsonREST

class DropdownLayout(_Component):
    """
    DropdownLayout component
    Lets users pick to see other components based on titles (works with "many" items). Similar to a :class:`~eureqa.analysis.components.tabbed_layout.TabbedLayout` but more suited to larger lists of items.

    For example::

        d = DropdownLayout(label="Choose an item", padding="1.0em")
        d.add_component(title="choice 1", content=HtmlBlock("This is item 1"))
        d.add_component(title="choice 2", content=HtmlBlock("This is item 2"))
        analysis.create_card(d)

    :param str label: The overall label
    :param str padding: padding, specified in ems. Example "0.5em"
    """

    _component_type_str = 'DROPDOWN_LAYOUT'

    def __init__(self, label=None, padding=None, _analysis=None, _component_id=None, _component_type=None):
        if label is not None:
            self._label = label

        if padding is not None:
            self._padding = padding

        # A place to put Components that don't have IDs yet,
        # until we're '_registered()'d with an Analysis so can go get IDs for them.
        self._queued_components = []

        super(DropdownLayout, self).__init__(_analysis=_analysis, _component_id=_component_id, _component_type=_component_type)


    @property
    def label(self):
        """Text label for the Dropdown box

        :rtype: str
        """
        return self._label

    @label.setter
    def label(self, val):
        self._label = val
        self._update()


    @property
    def padding(self):
        """Padding (spacing) for this DropdownLayout, expressed in ems. Example "0.5em"

        :rtype: int
        """
        return self._padding

    @padding.setter
    def padding(self, val):
        self._padding = val
        self._update()


    class _TitledComponent(object):
        """Internal class: Representation of one element in the dropdown

        :param str title: Title of the Component in the dropdown list
        :param eureqa.analysis.components._Component component: The Component
        """
        def __init__(self, title, component):
            self.title = title
            self.component = component

        def _to_json(self):
            return { "value": self.title,
                     "component_id": getattr(self.component, "_component_id", None) }

    @property
    def components(self):
        """Internal: Set of components represented in the dropdown

        :return: list() of :class:`~eureqa.analysis.components.base._Component`
        """
        if not hasattr(self, "_components_cache"):
            self._components_cache = [_Component._get(_analysis=self._analysis,
                                        _component_id=x["component_id"]) for x in self._components]

        return list(self._components_cache)

    def _ensure_components_list(self):
        if not hasattr(self, "_components"):
            self._components = []
            self._components_cache = []

    def add_component(self, title, component=None, content=None):
        """
        Add a Component to the DropdownLayout

        :param str title: Title of the Component in the dropdown list
        :param _Component component: The Component to add to the dropdown list
        :param _Component content: Deprecated, Do not use
        """
        if hasattr(self, "_analysis"):
            component._associate(self._analysis)

        #content remains for backwards compatibility
        #make sure only one is set and use that
        if (component is None) == (content is None):
            raise Exception("Incorrect arguments to add_component. Either component or content must be set, but not both.")

        if content is not None:
            component = content

        # If we don't have an Analysis yet, queue this Component to be saved later
        if not hasattr(self, "_analysis"):
            self._queued_components.append(DropdownLayout._TitledComponent(title, component))
            return

        self._ensure_components_list()
        self._components.append(DropdownLayout._TitledComponent(title, component)._to_json())
        self._components_cache.append(component)

    def _walk_children(self):
        if hasattr(self, "_components"):
            for c in self.components:
                yield c
        for c in self._queued_components:
            yield c.component

    def _register(self, analysis):
        if len(self._queued_components) != 0:
            self._ensure_components_list()
        for c in self._queued_components:
            self._components.append(c._to_json())
            self._components_cache.append(c.component)

        self._queued_components = []

        super(DropdownLayout, self)._register(analysis)

    def _associate_with_table(self, analysis):
        for c in self._queued_components:
            self._ensure_components_list()
            self._components += c.component._associate_with_table(analysis)

        self._queued_components = []

        return super(DropdownLayout, self)._associate_with_table(analysis) + self._components

    def _fields(self):
        return super(DropdownLayout, self)._fields() + [ 'label', 'padding', 'components' ]



# Copyright (c) 2017, Nutonian Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the Nutonian Inc nor the
#     names of its contributors may be used to endorse or promote products
#     derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL NUTONIAN INC BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


class _Organization:
    def __init__(self, eureqa, name):
        self._eureqa = eureqa
        self.name = name

    def create_user(self, user_email, password, role, first_name, last_name):
        key_url = '/api/v2/%s/auth/signup_key' % self.name
        key_request = {'organization': self.name, 'total_uses': 1, 'roles': ['%s' % role]}
        self._eureqa._session.report_progress('Creating signup_key for role \'%s\'.' % role)
        key = self._eureqa._session.execute(key_url, 'POST', args=key_request)

        body = user_request = {
            'firstname': first_name,
            'lastname': last_name,
            'username': user_email,
            'useremail': user_email,
            'password': password,
            'signup_key': key['signup_key']}
        self._eureqa._session.report_progress('Signing up user: \'%s\'.' % user_email)
        self._eureqa._session.execute('/api/v2/auth/signup', 'POST', args=user_request)

    def _delete_user(self, user_email):
        user_url = '/api/v2/auth/user/%s' % user_email
        self._eureqa._session.report_progress('Deleting user \'%s\'.' % user_email)
        self._eureqa._session.execute(user_url, 'DELETE')

    def _make_user_sys_admin(self, user_email):
        user_url = '/api/v2/auth/user/%s' % user_email
        self._eureqa._session.report_progress('Making user \'%s\' into sysadmin.' % user_email)
        user = self._eureqa._session.execute(user_url, 'GET')
        user["sysadmin"] = True
        self._eureqa._session.execute(user_url, 'POST', args=user)

    def _make_user_user_admin(self, user_email):
        user_url = '/api/v2/auth/user/%s' % user_email
        self._eureqa._session.report_progress('Making user \'%s\' into user_admin.' % user_email)
        user = self._eureqa._session.execute(user_url, 'GET')
        user["user_admin"] = True
        self._eureqa._session.execute(user_url, 'POST', args=user)

    def _set_search_limit(self, limit):
        org = self._eureqa._session.execute('/api/v2/organizations/%s' % self.name, 'GET')
        org['limits']['max_searches'] = limit
        self._eureqa._session.execute('/api/v2/organizations/%s' % self.name, 'POST', args=org)




from pprint import pformat
from itertools import izip
from copy import deepcopy


class _JsonREST(object):
    """
    Internal helper class.
    For objects that represent server-side state.

    Each subclass of '_JsonREST' represents a RESTful object on the backend server.

    Say you have a RESTful object that's accessible from:
    /users/:username/favorite_puppies/:puppy_id
    And whose JSON structure looks like:
    {
        "name": (...),
        "fur_color": (...),
        "height": (...)
    }

    Then, to use _JsonREST, you would create a subclass as follows:

    class FavoritePuppy(_JsonREST):
        def __init__(self, eureqa, username, _id=None):
            super(FavoritePuppy, self).__init__(eureqa)  # JsonREST needs a valid Eureqa connection
            self._username = username
            self._puppy_name = puppy_name
            if _id:
                self._id = _id
        def _directory_endpoint(self):
            return "/users/%s/favorite_puppies" % (self._username)
        def _object_endpoint(self):
            return "/users/%s/favorite_puppies/%s" % (self._username, self._id)
        def _fields(self):
            return ["id", "name", "fur_color", "height"]

    Then you can do:
    puppy = FavoritePuppy(eureqa, "jlpicard")
    print puppy._name           # "Poodles"
    print puppy._fur_color      # white
    print puppy._height         # 3
    puppy._create()             # Puppy is now saved on the server

    puppy._height = 4
    puppy._update()             # Server now stores a height of 4

    puppy_copy = FavoritePuppy.get(eureqa, "jlpicard", puppy._id)
    assert puppy == puppy_copy  # Should be the same

    puppy._delete()             # Puppy is no longer stored on the server

    And so on; see the docstrings on the functions below for details.

    Fields are preceded with underscores.  The expectation is that, for any public API,
    accessor properties will be written that control access and that provide autocomplete and docstrings.

    For a guide on how to convert existing classes to use this class, see README_JSONREST.txt
    """

    def __init__(self, eureqa):
        self._eureqa = eureqa

    ##########
    # All subclasses must implement the following methods
    ##########
    def _object_endpoint(self):
        """
        Compute and return a URL-path representing the current object.
        For example, '"/analyses/%s" % self._analysis_id'.
        """
        raise NotImplementedError

    def _directory_endpoint(self):
        """
        Return a URL-path representing the path containing these objects.
        (For POST'ing new object or GET'ing a list of all objects.)
        For example, '"/analyses"'.
        """
        raise NotImplementedError

    def _fields(self):
        """
        List (or other iterable) of field names to expect from the backend JSON,
        that map 1:1 to private properties on the current object.
        If this object has any fields that need special handling, they should
        be handled by overriding '_to_json()' and '_from_json()'.
        For example, '[ "id", "title", "description", "variables", "stuff", "more_things" ]'
        """
        raise NotImplementedError

    ##########
    # Default implementations (can optionally be overridden)
    ##########

    def _to_json(self):
        """ Convert this object into a tree of primitive JSON-compatible Python types """
        ret = {}
        for field, private_field in izip(self._fields(),
                                         self._private_fields()):
            if hasattr(self, private_field):
                ret[field] = getattr(self, private_field)
        return ret

    def _from_json(self, body):
        """
        Load the output of '_to_json()' into this object
        :param dict body: The output of '_to_json()'
        """
        for field, private_field in izip(self._fields(),
                                         self._private_fields()):
            if field in body:
                setattr(self, private_field, body.get(field))

    def _allow_request_object(self):
        """
        Returns True if it's currently ok to hit the REST endpoint.
        Should return False for partially-constructed objects.
        Note that HTTP requests will be IGNORED when this returns False!
        """
        try:
            self._object_endpoint()
            return True
        except:
            return False

    def _allow_request_directory(self):
        """
        Returns True if it's currently ok to hit the REST endpoint.
        Should return False for partially-constructed objects.
        Note that HTTP requests will be IGNORED when this returns False!
        """
        try:
            self._directory_endpoint()
            return True
        except:
            return False

    ##########
    # Provided helper methods (not intended to be overridden)
    ##########

    @classmethod
    def _construct_from_json(cls, body, *args, **kwargs):
        """
        Construct a new instance of this class.  Populate it with the contents of 'body'.
        :param dict body: The output of '_to_json()', to be used to populate the data in the new instance
        :return: Constructed instance of this class
        """
        ret = cls(*args, **kwargs)
        ret._from_json(body)
        return ret

    def _create(self):
        """ Construct the current object on the server """
        if not self._allow_request_directory():
            return

        endpoint = self._directory_endpoint()
        new_body = self._eureqa._session.execute(endpoint, 'POST', self._to_json())
        self._from_json(new_body)

    def _update(self):
        """ Update the current object on the server """
        if not self._allow_request_object():
            return

        endpoint = self._object_endpoint()
        new_body = self._eureqa._session.execute(endpoint, 'PUT', self._to_json())
        self._from_json(new_body)

    def _delete(self):
        """ Delete the current object on the server """
        if not self._allow_request_object():
            return

        endpoint = self._object_endpoint()
        self._eureqa._session.execute(endpoint, 'DELETE')

    def _get_self_json(self):
        """
        Fetch the JSON representation of the current object from the server
        :return: dict representing the JSON-serializable form of the current object
        """
        endpoint = self._object_endpoint()
        new_body = self._eureqa._session.execute(endpoint, 'GET')
        return new_body

    def _get_self(self):
        """
        Re-fetch the current object from the server.
        Mutates this object in place.
        :return: self
        """
        self._from_json(self._get_self_json())
        return self

    @classmethod
    def _get(cls, *args, **kwargs):
        """
        Fetch the specified object from the server.
        Keyword arguments should specify all fields used by '_object_endpoint()',
        as well as any arguments that this class's constructor requires
        in order to connect to the Eureqa server.
        :return: Instance of the current class describing the specified object
        """
        self = cls(*args, **kwargs)
        return cls._construct_from_json(self._get_self_json(), *args, **kwargs)

    def _get_all_json_from_self(self):
        """
        Fetch a JSON representation of all objects in the same directory as this object.
        :return: list() of dicts
        """
        endpoint = self._directory_endpoint()
        new_body_list = self._eureqa._session.execute(endpoint, 'GET')
        return new_body_list

    def _get_all_from_self(self):
        """
        Fetch a list of all objects in the same directory as this object.
        :return: list() of objects of the current type
        """
        new_body_list = self._get_all_json_from_self()

        new_obj_list = []
        for new_body in new_body_list:
            new_obj = self._construct_from_json(new_body)
            new_obj_list.append(new_obj)
        return new_obj_list

    @classmethod
    def _get_all(cls, *args, **kwargs):
        """
        Get all objects in the specified directory.
        Keyword arguments should specify all fields used by '_directory_endpoint()',
        as well as any arguments that this class's constructor requires
        in order to connect to the Eureqa server.
        :return: list() of instances of the current class
        """
        self = cls(*args, **kwargs)
        return [cls._construct_from_json(x, *args, **kwargs)
                for x in self._get_all_json_from_self()]

    def __eq__(self, other):
        return self._to_json() == other._to_json()

    def __ne__(self, other):
        return not (self == other)

    def __hash__(self):
        return hash(repr(self._to_json()))

    def __repr__(self):
        kwargs_list = ["%s=%s" % (key, repr(val))
                       for key, val in sorted(self._to_json().items())]
        return "%s(%s)" % (self.__class__.__name__, ", ".join(kwargs_list))

    def __str__(self):
        return pformat(self._to_json(), indent=4)

    ##########
    # Utility methods (for us, not for child classes)
    ##########

    def _private_fields(self):
        # Carefully use iterators to represent
        return ('_' + f for f in self._fields())



from eureqa.analysis.components.base import _Component
from eureqa.utils.jsonrest import _JsonREST


class Layout(_Component):
    """Generic layout within an Analysis

    A generic grid layout composed of rows and columns sized by
    fractions (`1`, `1/2`, `1/4`, etc). Layouts can be nested within
    other layouts to create any type of grid needed.

    For example::

       layout = Layout()
       layout = Layout(borders=True)                                       # create a layout where borders are displayed
       layout.add_component(HtmlBlock(html="<h1>Hybrid Performance</h1>")) # add an item that is an entire row
       layout.add_row()                                                    # start a new row. newly added components will be placed into the new row.
       layout.add_component(MagnitudeBar(value=-0.22), "1/3")              # add a magnutide bar on the left 1/3 of the screen
       layout.add_component(MagnitudeBar(value=-0.22), "1/3", True)        # add a magnutide bar with dark background enabled
       layout.add_component(HtmlBlock(html="Variables...."), "1/2")        # add a text panel in the next 1/2 of the screen
       analysis.create_card(layout)                                        # Add this Layout to an Analysis

    :param bool borders: If true, render visible borders on the layout component
    :param list rows: (optional) List of :class:`~eureqa.analysis.components.Layout.Row` objects to add to this class initially

    """

    class _GridItem(object):
        """ Internal implementation class """
        def __init__(self, component=None, grid_unit="1", dark_background=False, analysis=None):
            self.grid_unit = grid_unit
            self.dark_background = dark_background
            self._component = component
            if component is not None and hasattr(component, "_component_id"):
                self._component_id = component._component_id
            self._analysis = analysis

        def _to_json(self):
            ret = {
                "grid_unit": self.grid_unit,
                "dark_background": self.dark_background
            }
            if hasattr(self.component, "_component_id"):
                ret["component_id"] = self.component._component_id
            elif hasattr(self, "_component_id"):
                ret["component_id"] = self._component_id
            return ret

        def _from_json(self, body):
            if "grid_unit" in body:
                self.grid_unit = body["grid_unit"]
            if "dark_background" in body:
                self.dark_background = body["dark_background"]
            if "component_id" in body:
                self._component_id = body["component_id"]

        @property
        def component(self):
            # This is a bunch of code to implement
            # "if someone asks for this component and we don't have the component
            # but we do have its ID, go fetch the component on demand and store a
            # local copy for future use"
            # in a way that's backwards compatible with the old implementation that
            # had 'component' as a raw field on these objects.
            self._ensure_component()
            return self._component

        @component.setter
        def component(self, val):
            if hasattr(self, "_analysis"):
                val._associate(self._analysis)
            self._component = val
            if hasattr(self._component, "_component_id"):
                self._component_id = self._component._component_id

        @component.deleter
        def component(self):
            del self._component

        def _ensure_component(self):
            if self._component is None :
                self._component = _Component._get(_analysis=self._analysis,
                                                  _component_id=getattr(self, "_component_id", None))

        def _register(self, analysis):
            self._analysis = analysis
            self._ensure_component()

        def _associate_with_table(self, analysis):
            self._analysis = analysis
            self._ensure_component()
            return self.component._associate_with_table(analysis)

    class _Row(object):
        """ Internal implementation class """
        def __init__(self, analysis=None, grid_items=None):
            self.grid_items = grid_items if grid_items else []
            self._analysis = analysis

        def create_card(self, component, grid_unit="1", dark_background=False):
            self.grid_items.append(Layout._GridItem(component, grid_unit, dark_background, self._analysis))
            return self.grid_items[-1]

        def _to_json(self):
            for item in self.grid_items:
                if not hasattr(item.component, "_component_id"):
                    # Return a partial object to the server.
                    # Used to indicate that we want a partial response, ie.,
                    # we want an ID for this Layout but we don't know all of its childrens' IDs yet
                    return []
            return [item._to_json() for item in self.grid_items]

        def _from_json(self, val):
            if len(val) == len(self.grid_items):
                # Overwrite our existing objects in place.
                # Preserve any state that the server may have not sent back.
                for new_json, grid_item in zip(val, self.grid_items):
                    grid_item._from_json(new_json)
            elif len(val) == 0:
                pass  # Also preserve state in this case
            else:
                self.grid_items = []
                for item_json in val:
                    item = Layout._GridItem(analysis=self._analysis)
                    item._from_json(item_json)
                    self.grid_items.append(item)

        def _walk_children(self):
            for item in self.grid_items:
                yield item.component

        def _register(self, analysis):
            for item in self.grid_items:
                item._register(analysis)

        def _associate_with_table(self, analysis):
            return [item._associate_with_table(analysis) for item in self.grid_items]

        def __str__(self):
            return "_Row(%s)" % repr(self.grid_items)

    _component_type_str = 'GENERIC_LAYOUT'

    def __init__(self, rows=None, borders=False, _analysis=None, _component_id=None, _component_type=None):
        if rows is not None:
            self.rows = rows
        else:
            self.rows = []
        self._borders = borders

        super(Layout, self).__init__(_analysis=_analysis, _component_id=_component_id, _component_type=_component_type)

    @property
    def _current_row(self):
        # Break with convention:
        # 'self.rows' is not a property that points to 'self._rows
        if len(self.rows) == 0:
            self.rows = [ Layout._Row(analysis=getattr(self, "_analysis", None)) ]

        return self.rows[-1]

    @property
    def borders(self):
        return self._borders

    def add_row(self):
        self.rows.append(Layout._Row(analysis=getattr(self, "_analysis", None)))
        return self.rows[-1]

    def add_component(self, component, grid_unit="1", dark_background=False):
        return self._current_row.create_card(component, grid_unit, dark_background)

    def delete(self):
        for row in self.rows:
            for item in row.grid_items:
                item.component.delete()
        super(Layout, self).delete()

    def _to_json(self):
        ret = super(Layout, self)._to_json()
        ret["rows"] = [row._to_json() for row in self.rows]
        if len(ret["rows"]) == 0:
            del ret["rows"]
        return ret

    def _from_json(self, body):
        super(Layout, self)._from_json(body)

        # Handle 'rows'
        if "rows" not in body:
            return

        # If we're updating ourselves, preserve our cached Components
        # as a performance optimization.
        # If we get back no data from the server, likewise preserve our
        # cached Components; this means we asked for an ID but didn't have
        # our childrens' IDs yet.
        self_update = False
        if len(body["rows"]) == len(self.rows):
            self_update = True
            for body_row, self_row in zip(body["rows"], self.rows):
                if len(body_row) != 0 and len(body_row) != len(self_row.grid_items):
                    self_update = False
                    break

        if self_update:
            for body_row, self_row in zip(body["rows"], self.rows):
                self_row._from_json(body_row)

        # Else, create new row objects
        # These will be lazily downloaded from the server
        else:
            self.rows = []
            for row_json in body["rows"]:
                row = Layout._Row(analysis=getattr(self, "_analysis", None))
                row._from_json(row_json)
                self.rows.append(row)

    def _register(self, analysis):
        super(Layout, self)._register(analysis)
        for row in self.rows:
            row._register(analysis)

    def _fields(self):
        # 'rows' is handled in _to_json
        return super(Layout, self)._fields() + [ "borders" ]

    def _walk_children(self):
        for row in self.rows:
            for c in row._walk_children():
                yield c

    def _associate_with_table(self, analysis):
        components_json = []
        for row in self.rows:
            components_json += row._associate_with_table(analysis)
        return super(Layout, self)._associate_with_table(analysis) + components_json



#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL NUTONIAN INC BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import json

from parameter_value import ParameterValue
from text_parameter_value import TextParameterValue
from data_source_parameter_value import DataSourceParameterValue
from top_level_model_parameter_value import TopLevelModelParameterValue
from variable_parameter_value import VariableParameterValue
from data_file_parameter_value import DataFileParameterValue
from combo_box_parameter_value import ComboBoxParameterValue
from numeric_parameter_value import NumericParameterValue

from collections import OrderedDict

class ParametersValues:
    """Analysis template parameters values

    :param list[Parameter] parameters: The list of parameter values for the template, whether text, variable or datasource value.
    """

    def __init__(self, parameters=None, _execution=None):
        """Initializes a new instance of the ~ParametersValues class
        :param list ~Parameter parameters: the list of parameter values for the template, whether text, variable or datasource value
        """
        self.parameters = parameters if isinstance(parameters, dict) else OrderedDict((p.id, p) for p in parameters) if parameters else parameters
        self._execution = _execution
                
    def _to_json(self):
        body = {}
        if self.parameters is not None and len(self.parameters) > 0:
            body['parameters'] = [x._to_json() for x in self.parameters.itervalues()]
        return body

    def __str__(self):
        return json.dumps(self._to_json(), indent=4)

    _parameter_map = {
        'combo_box': ComboBoxParameterValue,
        'data_file': DataFileParameterValue,
        'datasource': DataSourceParameterValue,
        'numeric': NumericParameterValue,
        'top_level_model': TopLevelModelParameterValue,
        'text': TextParameterValue,
        'text_multiline': TextParameterValue,
        'variable': VariableParameterValue
    }

    @classmethod
    def _from_json(cls, eureqa, body, execution=None):
        ret = ParametersValues()
        if not 'parameters' in body: return ret

        parameters = []
        for element in body['parameters']:
            parameter = ParameterValue(None, None, None)
            ParameterValue._from_json(parameter, element, execution=execution)
            if not parameter._type in cls._parameter_map: raise Exception('Invalid analysis template parameter value type \'%s\' for parameter \'%s\' with value \'%s\'' %(parameter._type, parameter.id, parameter.value))
            parameter = cls._parameter_map[parameter._type]._from_json(eureqa, element, execution=execution)
            parameters.append(parameter)
        ret.parameters = OrderedDict((p.id, p) for p in parameters)
        return ret



# Copyright (c) 2017, Nutonian Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the Nutonian Inc nor the
#     names of its contributors may be used to endorse or promote products
#     derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL NUTONIAN INC BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from eureqa.analysis.components.base import _Component

class TextBlock(_Component):
    """Contains free-form user-specified text, formatted using
    markdown. *Deprecated*. Use :class:`~eureqa.analysis.components.html_block.HtmlBlock` instead.

    :param str text: Body of the card.
    :param str description: alias for text (backwards compatible)

    :var str eureqa.analysis.components.TextBlock.text: Body of the card.
    """

    _component_type_str = 'ANALYSIS_TEXT_BLOCK'

    def __init__(self, text='', _analysis=None, _component_id=None, _component_type=None, description=None):
        if not text and description:
            "For backward compatibility"
            self._text = description
        else:
            self._text = text

        super(TextBlock, self).__init__(_analysis=_analysis, _component_id=_component_id, _component_type=_component_type)

    @property
    def text(self):
        """Markdown-formatted text contents of this component

        :rtype: str"""
        return self._text

    @text.setter
    def text(self, val):
        self._text = val
        self._update()

    def _fields(self):
        return super(TextBlock, self)._fields() + [ 'text' ]



# Copyright (c) 2017, Nutonian Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the Nutonian Inc nor the
#     names of its contributors may be used to endorse or promote products
#     derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL NUTONIAN INC BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from eureqa.analysis.components.base import _Component
from eureqa.analysis.analysis_file import AnalysisFile

import os.path

class Image(_Component):
    """
    Image component
    Represents an image which can be used in an Analysis.

    For example::

        i = Image('camel.png')
        analysis.create_html_card('Image here: {0}'.format(analysis.html_ref(i)))

    :param str image_path: path to the image file
    :param int width:  (optional) a specific width to display the image with
    :param int height: (optional) a specific height to display the image with
    """

    _component_type_str = 'IMAGE'

    _SUPPORTED_IMAGE_TYPES = ['.svg', '.jpg', '.jpeg', '.png', '.gif']

    def __init__(self, image_path=None, width=None, height=None,
                 _analysis=None, _component_id=None, _component_type=None):
        if image_path is not None:
            if not os.path.isfile(image_path):
                raise Exception('Image file located at "' + image_path + '" does not exist')
            self._image_path = image_path
            self._image_file_name = os.path.basename(self._image_path)
            image_file_ext = os.path.splitext(self._image_file_name)[-1].lower()
            if image_file_ext not in self._SUPPORTED_IMAGE_TYPES:
                raise Exception('Image file "' + self._image_file_name + '" is not supported. ' +
                                'Valid types are ' + ', '.join(self._SUPPORTED_IMAGE_TYPES))

        if width is not None:
            self._width = width
        if height is not None:
            self._height = height

        super(Image, self).__init__(_analysis=_analysis, _component_id=_component_id, _component_type=_component_type)

    @property
    def image_url(self):
        return getattr(self, '_image_url', None)

    @property
    def image_file_id(self):
        return getattr(self, '_image_file_id', None)

    @property
    def width(self):
        return getattr(self, '_width', None)

    @property
    def height(self):
        return getattr(self, '_height', None)

    @property
    def file(self):
        """
        :rtype: eureqa.analysis.analysis_file.AnalysisFile
        """
        if hasattr(self, "_image_file_id") and hasattr(self, "_analysis") and not hasattr(self, "_file"):
            self._file = AnalysisFile(self._analysis, self._image_file_id)
        return getattr(self, "_file", None)

    def _upload_image(self, analysis):
        with open(self._image_path, 'rb') as image_file:
            self._file = AnalysisFile.create(analysis, image_file, filename=self._image_file_name)
            self._image_file_id = self._file._file_id
        self._image_url = ("/api/%s/analysis/%s/files/%s" % (analysis._organization, analysis._id, self._image_file_id))

    def _pre_associate(self, analysis):
        if hasattr(self, '_image_path') and not hasattr(self, '_image_url'):
            self._upload_image(analysis)
            # ideally, image_url should be returned by the app server, but if we do a
            # POST to the app server here, it requires another network operation and
            # the component goes into analysis' main component list, which isn't
            # desired for components used in table

    def _fields(self):
        return super(Image, self)._fields() + [ "image_url", "image_file_id", "width", "height" ]
